{======================================================================}
{ COMM-IO.INC: Turbo Communucations Port Routines                      }

Type
  ComPortType     = 1..2;           { Two available comm ports }
  BaudType        = 300..9600;
  ParityType      = (Space, Odd, Mark, Even, None);
  DataBitsType    = 5..8;
  StopBitsType    = 1..2;

  VectorType      = record
    Offset      : integer;          { Instruction pointer }
    Segment     : integer;          { Code segment }
  End;

  RegistersType   = record
    Case Integer of
      1 : (AX,BX,CX,DX,BP,SI,DI,DS,ES,Flags : Integer);
      2 : (AL,AH,BL,BH,CL,CH,DL,DH : Byte);
    End;

Const
  BufferSize      = 1024;           { Size of receive buffer in bytes }

                          { Offsets into base addresses }

  THR             = $00;  { Transmitter Holding Register          }
  RBR             = $00;  { Receiver Buffer Register              }
  DLL             = $00;  { Divisor Latch Least Significant Byte  }
  DLM             = $01;  { Divisor Latch Most Significant Byte   }
  IER             = $01;  { Interrupt Enable Register             }
  IIR             = $02;  { Interrupt Identification Register     }
  LCR             = $03;  { Line Control Register                 }
  MCR             = $04;  { Modem Control Register                }
  LSR             = $05;  { Line Status Register                  }
  MSR             = $06;  { Modem Status Register                 }

  IMlist : Array[ComPortType] of Integer = ($EF,$F7);
  IVlist : Array[ComPortType] of Integer = ($000C,$000B);

  PICCMD          = $20;            { 8259 control port }
  PICMSK          = $21;            { 8259 control port }

  OurDS : Integer = -1;             { Used during receive interrupt }

Var
  ComPort         : ComPortType;    { Active communications port }
  Baud            : BaudType;       { Current baud rate }
  Parity          : ParityType;     { Parity }
  DataBits        : DataBitsType;   { Data bits }
  StopBits        : StopBitsType;   { Stop bits }
  BaseAddress     : Array[ComPortType] of Integer absolute $0040:$0000;
  Buffer          : Array[0..BufferSize] of Char; { buffer storage }
  Top, Bottom     : Integer;                      { buffer pointers }
  OldVector       : VectorType;     { Old interrupt vector }
  Base            : Integer;        { Base address of ports }

{======================================================================}
Procedure SetVector(Vector:Integer; Var NewVector:VectorType);

Var
  Registers     : RegistersType;

Begin
  Inline($FA);                      { CLI disable interrupts }
  With Registers do
    Begin
      AH := $35;
      AL := Vector;
      MsDos(Registers);             { DOS function hex 35 - get vector }
      DS := NewVector.Segment;
      DX := NewVector.Offset;
      NewVector.Segment := ES;
      NewVector.Offset := BX;
      AH := $25;
      AL := Vector;
      MsDos(Registers);             { DOS function hex 25 - set vector }
    End;
    Inline($FB);                    { STI reenable interrupts }
  End; {endwith}

{======================================================================}
Procedure ReceiveInterrupt;

Var
  Dummy         : Array[1..8] of Integer;       { Room for PUSHes }
  LSRstatus     : Byte;
  IIRstatus     : Byte;

Begin
  Inline($50/$53/$51/$52/$57/$56/$06);  { Push Registers and CLI }
  Inline($1E);                          { PUSH DS }
  Inline($2E/$8E/$1E/OurDS);            { MOV DS,CS:OurDS }
  IIRstatus := Port[Base+IIR];          { Check the interrupt call }
  If (IIRstatus and $06) = $04 then
    If (Succ(Top) mod BufferSize) <> Bottom then  { Store the character }
      Begin
        Buffer[Top] := Chr(Port[Base+RBR]);
        Top := Succ(Top) mod BufferSize;
      End;
  If (IIRstatus and $06) = $06 then     { Ignore an overrun error }
    LSRstatus := Port[Base+LSR];
  Port[PICCMD] := $20;                  { Send end of interrupt to the 8259 }
  Inline($1F);                          { POP DS }
  Inline($07/$5E/$5F/$5A/$59/$5B/$58);  { Pop other registers }
  Inline($89/$EC);                      { MOV SP,BP }
  Inline($5D);                          { POP BP }
  Inline($CF);                          { IRET }
End;

{======================================================================}
Procedure OpenComPort;

{ ComPort, Baud, Parity, DataBits, and StopBits MUST be set first!     }

Var
  Dummy     : Byte;
  Counter   : Integer;

Begin
  OurDS := Dseg;                { used during the interrupt }

  With OldVector do             { swap interrupt vector }
    Begin
      Segment := Cseg;
      Offset := Ofs(ReceiveInterrupt);
    End;
  SetVector(IVlist[ComPort],OldVector);

  Dummy := Port[Base+LSR];      { read any garbage out of ports }
  Dummy := Port[Base+RBR];
  Dummy := Port[Base+IIR];
  Dummy := Port[Base+MSR];

  Base := BaseAddress[ComPort]; { set base address }
  Top := 0;                     { reset receive buffer }
  Bottom := 0;

  Port[PICMSK] := Port[PICMSK] and IMlist[ComPort];

  Port[Base+IER] := $05;        { enable interrupts }
  Port[Base+MCR] := $0B;        { set RTS, DTR and OUT2 }
  Dummy := Port[Base+LSR];      { reset the Line Status Register }
  Bottom := Top;                { purge any garbage characters }
End;

{======================================================================}
Procedure CloseComPort;

{ This Procedure MUST be called once the port is opened or the computer}
{ may lock up requiring a complete power down (a CPU hardware reset is }
{ NOT going to do the trick, since the modem is "hung".                }

Begin
  Port[PICMSK] := Port[PICMSK] or ($FF-IMlist[ComPort]);
  Port[Base+IER] := $00;
  Port[Base+MCR] := Port[Base+MCR] and $F7;
  SetVector(IVlist[ComPort],OldVector);
End;

{======================================================================}
Function Carrier:Boolean;

{ This function will return 'true' if a carrier is present.            }

Begin
  Carrier := (Port[Base+MSR] and $80) <> $00;
End;

{======================================================================}
Procedure WriteCom(DataOut:Char);

{ Sends a character out to the modem.                                  }

Begin
  Repeat until (Port[Base+LSR] and $20) <> $00;
  Port[Base+THR] := Ord(DataOut);
End;
{===========================[ END OF FILE ]============================}